@@ -17,7 +17,10 @@
 #ifndef _NUMA_H
 #define _NUMA_H 1
 
-/* Simple NUMA poliy library */
+/* allow an application to test for the current programming interface: */
+#define LIBNUMA_API_VERSION 2
+
+/* Simple NUMA policy library */
 
 #include <stddef.h>
 #include <string.h>
@@ -58,14 +61,71 @@ void copy_bitmask_to_nodemask(struct bitmask *, nodemask_t *);
 void copy_bitmask_to_bitmask(struct bitmask *, struct bitmask *);
 
 /* compatibility for codes that used them: */
-static inline void nodemask_zero(struct bitmask *mask)
+
+static inline void nodemask_zero(nodemask_t *mask)
+{
+	struct bitmask tmp;
+
+	tmp.maskp = (unsigned long *)mask;
+	tmp.size = sizeof(nodemask_t) * 8;
+	numa_bitmask_clearall(&tmp);
+}
+
+static inline void nodemask_zero_compat(nodemask_t *mask)
 { 
-	numa_bitmask_clearall(mask);
+	struct bitmask tmp;
+
+	tmp.maskp = (unsigned long *)mask;
+	tmp.size = sizeof(nodemask_t) * 8;
+	numa_bitmask_clearall(&tmp);
 } 
 
-static inline int nodemask_equal(struct bitmask *a, struct bitmask *b)
+static inline void nodemask_set_compat(nodemask_t *mask, int node)
+{
+	mask->n[node / (8*sizeof(unsigned long))] |=
+		(1UL<<(node%(8*sizeof(unsigned long))));
+}
+
+static inline void nodemask_clr_compat(nodemask_t *mask, int node)
+{
+	mask->n[node / (8*sizeof(unsigned long))] &=
+		~(1UL<<(node%(8*sizeof(unsigned long))));
+}
+
+static inline int nodemask_isset_compat(const nodemask_t *mask, int node)
+{
+	if ((unsigned)node >= NUMA_NUM_NODES)
+		return 0;
+	if (mask->n[node / (8*sizeof(unsigned long))] &
+		(1UL<<(node%(8*sizeof(unsigned long)))))
+		return 1;
+	return 0;
+}
+
+static inline int nodemask_equal(const nodemask_t *a, const nodemask_t *b)
+{
+	struct bitmask tmp_a, tmp_b;
+
+	tmp_a.maskp = (unsigned long *)a;
+	tmp_a.size = sizeof(nodemask_t) * 8;
+
+	tmp_b.maskp = (unsigned long *)b;
+	tmp_b.size = sizeof(nodemask_t) * 8;
+
+	return numa_bitmask_equal(&tmp_a, &tmp_b);
+}
+
+static inline int nodemask_equal_compat(const nodemask_t *a, const nodemask_t *b)
 { 
-	return numa_bitmask_equal(a, b);
+	struct bitmask tmp_a, tmp_b;
+
+	tmp_a.maskp = (unsigned long *)a;
+	tmp_a.size = sizeof(nodemask_t) * 8;
+
+	tmp_b.maskp = (unsigned long *)b;
+	tmp_b.size = sizeof(nodemask_t) * 8;
+
+	return numa_bitmask_equal(&tmp_a, &tmp_b);
 } 
 
 /* NUMA support available. If this returns a negative value all other function
@@ -101,6 +161,8 @@ extern struct bitmask *numa_no_nodes_ptr;
 /* Source compatibility */
 extern nodemask_t numa_no_nodes;
 
+extern char *nodes_allowed_list;
+
 /* Only run and allocate memory from a specific set of nodes. */
 void numa_bind(struct bitmask *nodes);
 
@@ -208,12 +270,12 @@ static inline void numa_free_cpumask(struct bitmask *b)
 	numa_bitmask_free(b);
 }
 
-/* set up to represent the cpus available to the current task */
-struct bitmask *numa_all_cpus;
-
 /* Convert node to CPU mask. -1/errno on failure, otherwise 0. */
 int numa_node_to_cpus(int, struct bitmask *);
 
+/* report the node of the specified cpu. -1/errno on invalid cpu. */
+int numa_node_of_cpu(int cpu);
+
 /* Report distance of node1 from node2. 0 on error.*/
 int numa_distance(int node1, int node2);
 
@@ -250,7 +312,7 @@ struct bitmask *numa_parse_cpustring(char *);
 /*
  * The following functions are for source code compatibility
  * with releases prior to version 2.
- * Such codes should be compiled with VERSION1_COMPATIBILITY defined.
+ * Such codes should be compiled with NUMA_VERSION1_COMPATIBILITY defined.
  */
 
 static inline void numa_set_interleave_mask_compat(nodemask_t *nodemask)
@@ -387,9 +449,9 @@ static inline int numa_node_to_cpus_compat(int node, unsigned long *buffer,
 
 /*
  * To compile an application that uses libnuma version 1:
- *   add -DVERSION1_COMPATIBILITY to your Makefile's CFLAGS
+ *   add -DNUMA_VERSION1_COMPATIBILITY to your Makefile's CFLAGS
  */
-#ifdef VERSION1_COMPATIBILITY
+#ifdef NUMA_VERSION1_COMPATIBILITY
 #include <numacompat1.h>
 #endif
 
