@@ -49,6 +49,7 @@ struct option opts[] = {
 	{"strict", 0, 0, 't'},
 	{"shmmode", 1, 0, 'M'},
 	{"dump", 0, 0, 'd'},
+	{"dump-nodes", 0, 0, 'D'},
 	{"shmid", 1, 0, 'I'},
 	{"huge", 0, 0, 'u'},
 	{"touch", 0, 0, 'T'},
@@ -67,8 +68,8 @@ void usage(void)
 		"       numactl [--length length] [--offset offset] [--shmmode shmmode]\n"
 		"               [--strict]\n"
 		"               [--shmid id] --shm shmkeyfile | --file tmpfsfile\n"
-		"               [--huge] [--touch]\n" 
-		"               memory policy\n"
+		"               [--huge] [--touch] \n"
+		"               memory policy | --dump | --dump-nodes\n"
 		"\n"
 		"memory policy is --interleave, --preferred, --membind, --localalloc\n"
 		"nodes is a comma delimited list of node numbers or A-B ranges or all.\n"
@@ -199,40 +200,40 @@ static void print_distances(int maxnode)
 
 void print_node_cpus(int node)
 {
-	int len = 1;
 	int conf_cpus = numa_num_configured_cpus();
+	int i, err;
+	struct bitmask *cpus;
 
-	for (;;) { 
-		int i, err;
-		struct bitmask *cpus;
-
-		cpus = numa_bitmask_alloc(conf_cpus);
-		errno = 0;
-		err = numa_node_to_cpus(node, cpus);
-		if (err < 0) {
-			if (errno == ERANGE) {
-				len *= 2; 
-				continue;
-			}
-			break; 
-		}
-		for (i = 0; i < len*BITS_PER_LONG; i++) 
+	cpus = numa_bitmask_alloc(conf_cpus);
+	err = numa_node_to_cpus(node, cpus);
+	if (err >= 0) 
+		for (i = 0; i < conf_cpus; i++) 
 			if (numa_bitmask_isbitset(cpus, i))
 				printf(" %d", i);
-		break;
-	}
 	putchar('\n');
 }
 
 void hardware(void)
 { 
-	int i;
+	int i, numconfigurednodes=0;
 	int maxnode = numa_num_configured_nodes()-1;
-	printf("available: %d nodes (0-%d)\n", 1+maxnode, maxnode); 	
+
+	for (i = 0; i<=maxnode; i++)
+		if (numa_bitmask_isbitset(numa_all_nodes_ptr, i))
+			numconfigurednodes++;
+	if (nodes_allowed_list)
+		printf("available: %d nodes (%s)\n",
+			numconfigurednodes, nodes_allowed_list);
+	else
+		printf("available: %d nodes (0-%d)\n", maxnode+1, maxnode); 	
+		
 	for (i = 0; i <= maxnode; i++) { 
 		char buf[64];
 		long long fr;
 		unsigned long long sz = numa_node_size64(i, &fr); 
+		if (!numa_bitmask_isbitset(numa_all_nodes_ptr, i))
+			continue;
+
 		printf("node %d cpus:", i);
 		print_node_cpus(i);
 		printf("node %d size: %s\n", i, fmt_mem(sz, buf));
@@ -463,10 +464,18 @@ int main(int ac, char **av)
 			break;
 		case 'd': /* --dump */
 			if (shmfd < 0)
-				complain("Cannot do --dump without shared memory.\n");
+				complain(
+				"Cannot do --dump without shared memory.\n");
 			dump_shm();
 			do_dump = 1;
 			break;
+		case 'D': /* --dump-nodes */
+			if (shmfd < 0)
+				complain(
+			    "Cannot do --dump-nodes without shared memory.\n");
+			dump_shm_nodes();
+			do_dump = 1;
+			break;
 		case 't': /* --strict */
 			did_strict = 1;
 			numa_set_strict(1);
@@ -518,10 +527,11 @@ int main(int ac, char **av)
 	}
 
 	if (did_strict)
-		fprintf(stderr,"numactl: warning. Strict flag for process ignored.\n");
+		fprintf(stderr,
+			"numactl: warning. Strict flag for process ignored.\n");
 
 	if (do_dump)
-		usage_msg("cannot do --dump for process");
+		usage_msg("cannot do --dump|--dump-shm for process");
 
 	if (shmoption)
 		usage_msg("shm related option %s for process", shmoption);
